unit frmbase;

{$mode ObjFPC}

interface

uses
  Web, Classes, SysUtils;

Type
  EForms = Class(Exception);

  { TBaseForm }

  TBaseForm = class(TComponent)
  Public
    Class Function FormName : String; virtual;
    Class Function FormHTMLFileName : String; virtual;
    Class Procedure Register;
  end;
  TBaseFormClass = class of TBaseForm;

  TFormProcedure = Reference to procedure (sender : TObject; aForm : TBaseForm);

  { TFormManager }

  TFormManager = Class(TComponent)
  Private
    FCurrentForm : TBaseForm;
    FFormParent: TJSHTMLElement;
    FForms : TStringList;
    class var _instance : TFormManager;
  Public
    class constructor Init;
    Constructor Create(aOwner : TComponent); override;
    Destructor Destroy; override;
    Procedure RegisterForm(aClass : TBaseFormClass; const aName : String = ''; aHTMLFile : String = '');
    Procedure UnregisterForm(aName : string);
    Procedure ShowForm(aName : string; OnShow : TFormProcedure = nil);
    Property CurrentForm : TBaseForm Read FCurrentForm;
    Property FormParent : TJSHTMLElement Read FFormParent Write FFormParent;
    Class property Instance : TFormManager Read _Instance;
  end;

Function FormManager : TFormManager;

implementation

uses Rtl.TemplateLoader;

function FormManager: TFormManager;
begin
  Result:=TFormManager.Instance;
end;

Type

  { TFormRegistration }

  TFormRegistration = Class(TObject)
  private
    FFormClass: TBaseFormClass;
    FHTML: String;
    FName: string;
  public
    Constructor Create (aClass: TBaseFormClass; aName,aHTML : String);
    Property FormClass : TBaseFormClass Read FFormClass;
    Property Name : string Read FName;
    Property HTML : String Read FHTML;
  end;

{ TFormRegistration }


constructor TFormRegistration.Create(aClass: TBaseFormClass; aName,
  aHTML: String);
begin
  FFormClass:=aClass;
  FName:=aName;
  FHTML:=aHTML;
end;

{ TFormManager }

class constructor TFormManager.Init;

begin
  _instance:=TFormManager.Create(Nil);
end;

constructor TFormManager.Create(aOwner: TComponent);
begin
  inherited Create(aOwner);
  FForms:=TStringList.Create;
  FForms.OwnsObjects:=True;

end;

destructor TFormManager.Destroy;
begin
  inherited Destroy;
end;

procedure TFormManager.RegisterForm(aClass: TBaseFormClass;
  const aName: String; aHTMLFile: String);

Var
  N,H : String;


begin
  N:=aName;
  if N='' then N:=aClass.FormName;
  H:=aHTMLFile;
  if H='' then H:=aClass.FormHTMLFileName;
  FForms.AddObject(N,TFormRegistration.Create(aClass,N,H));
end;

procedure TFormManager.UnregisterForm(aName: string);

Var
  Idx : Integer;

begin
  Idx:=FForms.IndexOf(aName);
  if Idx<>-1 then
    FForms.Delete(Idx);
end;

Resourcestring
  SErrUnknownForm = 'Unknown form name : %s';
  SErrUnknownFormTemplate = 'Unknown form template';

procedure TFormManager.ShowForm(aName: string; OnShow: TFormProcedure);

Var
  Idx : Integer;
  Reg : TFormRegistration;

  Procedure ShowForm;

  begin
    If Assigned(FCurrentForm) then
      FreeAndNil(FCurrentForm);
    FFormParent.innerHTML:=GlobalTemplates.Templates['form:'+Reg.Name];
    FCurrentForm:=Reg.FFormClass.Create(Self);
    If Assigned(OnShow) then
      OnShow(Self,FCurrentForm);
  end;

  procedure FormFailed(Sender: TObject; const aTemplate, aError: String;
  aErrorcode: Integer);
  begin
    Writeln('Error loading form template',aTemplate,' : ',aError,' (Code:',aErrorCode,')');
  end;

  procedure HaveForm(Sender: TObject; const aTemplate: String);
  begin
    ShowForm;
  end;

begin
  Idx:=FForms.IndexOf(aName);
  if Idx=-1 then
    Raise EForms.CreateFmt(SErrUnknownForm,[aName]);
  Reg:=TFormRegistration(FForms.Objects[Idx]);
  if GlobalTemplates.Templates['form:'+Reg.Name]='' then
    GlobalTemplates.LoadTemplate('form:'+Reg.Name,Reg.HTML,@HaveForm,@FormFailed)
  else
    ShowForm;
end;

{ TBaseForm }

class function TBaseForm.FormName: String;

Var
  P : integer;

begin
  Result:=LowerCase(ClassName);
  if Result.StartsWith('tfrm') then
    Result:=Copy(Result,5,Length(Result)-4)
  else if Result.StartsWith('t') then
    Result:=Copy(Result,2,Length(Result)-1);
  if Result.EndsWith('form') then
    begin
    P:=Pos('form',Result);
    Result:=Copy(Result,1,P-1);
    end;
end;

class function TBaseForm.FormHTMLFileName: String;
begin
  Result:=FormName+'.html';
end;

class procedure TBaseForm.Register;
begin
  TFormManager.Instance.RegisterForm(Self,FormName,FormHTMLFileName);
end;

end.

